@@ -3,13 +3,20 @@ import json
 import logging
 import platform
 from datetime import datetime
-
+from aiortc import VideoStreamTrack
+from av import VideoFrame
 import pyaudio as pyaudio
 import socketio
 from aiortc import RTCPeerConnection, RTCRtpSender, RTCConfiguration, RTCIceServer, RTCSessionDescription, \
     MediaStreamTrack
 from aiortc.contrib.media import MediaPlayer, MediaRelay, MediaBlackhole
 import subprocess
+from queue import Queue
+import cv2
+from random import random
+import asyncio
+import collections
+import time
 
 pcs = set()
 relay = None
@@ -22,6 +29,57 @@ stream = p.open(format=pyaudio.paFloat32,
                 rate=48000,
                 output=True)
 
+video_tracklist = []
+
+
+class FPS:
+    def __init__(self, avarageof=50):
+        self.frametimestamps = collections.deque(maxlen=avarageof)
+
+    def __call__(self):
+        self.frametimestamps.append(time.time())
+        if len(self.frametimestamps) > 1:
+            return len(self.frametimestamps) / (self.frametimestamps[-1] - self.frametimestamps[0])
+        else:
+            return 0.0
+
+
+async def checkTerminatorQueue(opqueue):
+
+    while True:
+
+        data = await opqueue.get()
+        if data is not None:
+            for track in video_tracklist:
+                await track.write(data, 0)
+
+
+class TerminatorStreamTrack(VideoStreamTrack):
+    """
+    A video track that returns an animated flag.
+    """
+
+    def __init__(self):
+        super().__init__()  # don't forget this!
+        self.counter = 0
+        height, width = 480, 640
+        self.framerate = 0
+        self.nal_queue = asyncio.Queue(3)
+
+    async def write(self, frame, framerate=0):
+        self.framerate = framerate
+        if not self.nal_queue.full():
+            await self.nal_queue.put(frame)
+
+    async def recv(self):
+        pts, time_base = await self.next_timestamp()
+
+        frame = VideoFrame.from_ndarray(await self.nal_queue.get())
+        frame.pts = pts
+        frame.time_base = time_base
+        self.counter += 1
+        return frame
+
 
 def create_local_tracks(play_from, decode, device, rtbufsize):
     global relay, webcam
@@ -207,13 +265,15 @@ def channel_send(logger, channel, message):
 class GSPeerConnectionBroadcasterUniversalNew:
 
     @classmethod
-    async def create(cls, gsdbs, target, videostream=True, addChannel=None, oncommand=None):
-        self = GSPeerConnectionBroadcasterUniversal()
+    async def create(cls, gsdbs, target, videostream=True, addChannel=None, oncommand=None, videotrack=None):
+        self = GSPeerConnectionBroadcasterUniversalNew()
         self.addChannel = addChannel
         self.rtconfiList = []
+        self.target = target
         self.gsdbs = gsdbs
         self.videostream = videostream
         self.oncommand = oncommand
+        self.videotrack = videotrack
         if self.gsdbs.credentials["stunenable"]:
             self.rtconfiList.append(RTCIceServer(self.gsdbs.credentials["stunserver"]))
         if self.gsdbs.credentials["turnenable"]:
@@ -237,13 +297,12 @@ class GSPeerConnectionBroadcasterUniversalNew:
             await self.sio.emit("joined", self.sio.sid)
 
         @self.sio.event
-        async def broadcaster(id, description):
+        async def broadcaster(id, description, emitter):
 
             if len(self.rtconfiList) > 0:
                 pc = RTCPeerConnection(configuration=RTCConfiguration(self.rtconfiList))
             else:
                 pc = RTCPeerConnection()
-
             self.peerConnections[id] = pc
 
             @pc.on("track")
@@ -279,34 +338,38 @@ class GSPeerConnectionBroadcasterUniversalNew:
                 if video:
                     video_sender = pc.addTrack(video)
                     force_codec(pc, video_sender, "video/H264")
+            if self.videotrack:
+                terminatorviewtrack = TerminatorStreamTrack()
+                video_tracklist.append(terminatorviewtrack)
+                pc.addTrack(terminatorviewtrack)
 
-            if self.addChannel is not None:
-                sendchannel = pc.createDataChannel("terminatorview")
-                self.addChannel(sendchannel)
-                channel_log(self._logger, sendchannel, "-", "created by local party")
+            # if self.addChannel is not None:
+            #     sendchannel = pc.createDataChannel("terminatorview")
+            #     self.addChannel(sendchannel)
+            #     channel_log(self._logger, sendchannel, "-", "created by local party")
+            #
+            #     @sendchannel.on("open")
+            #     def on_open():
+            #         self._logger.info("datachannel open")
 
-                @sendchannel.on("open")
-                def on_open():
-                    self._logger.info("datachannel open")
-
-            @sendchannel.on("message")
-            def on_message(message):
-                def subprocess_cmd(command):
-                    process = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
-                    proc_stdout = process.communicate()[0].strip()
-                    # for line in proc_stdout.decode().split('\n'):
-                    channel_send(self._logger, sendchannel, json.dumps({"response": proc_stdout.decode()}))
-
-                channel_log(self._logger, sendchannel, "<", message)
-                if isinstance(message, str) and message.startswith("ping"):
-                    # pass
-                    # reply
-                    channel_send(self._logger, sendchannel, f'{{"ping":"pong","ts":"{datetime.utcnow()} UTC"}}')
-                if isinstance(message, str) and message.startswith('{"type'):
-                    # reply
-                    self.oncommand(self.gsdbs, json.loads(message))
-                if isinstance(message, str) and message.startswith('{"command'):
-                    subprocess_cmd(json.loads(message)["command"])
+            # @sendchannel.on("message")
+            # def on_message(message):
+            #     def subprocess_cmd(command):
+            #         process = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
+            #         proc_stdout = process.communicate()[0].strip()
+            #         # for line in proc_stdout.decode().split('\n'):
+            #         channel_send(self._logger, sendchannel, json.dumps({"response": proc_stdout.decode()}))
+            #
+            #     channel_log(self._logger, sendchannel, "<", message)
+            #     if isinstance(message, str) and message.startswith("ping"):
+            #         # pass
+            #         # reply
+            #         channel_send(self._logger, sendchannel, f'{{"ping":"pong","ts":"{datetime.utcnow()} UTC"}}')
+            #     if isinstance(message, str) and message.startswith('{"type'):
+            #         # reply
+            #         self.oncommand(self.gsdbs, json.loads(message))
+            #     if isinstance(message, str) and message.startswith('{"command'):
+            #         subprocess_cmd(json.loads(message)["command"])
 
             await pc.setLocalDescription(await pc.createOffer())
             await self.sio.emit("watcher", {"target": description,
@@ -344,5 +407,5 @@ class GSPeerConnectionBroadcasterUniversalNew:
             connectURL = self.gsdbs.credentials["signalserver"]
 
         await self.sio.connect(
-            f'{connectURL}?gssession={self.gsdbs.cookiejar.get("session")}.{self.gsdbs.cookiejar.get("signature")}{self.gsdbs.credentials["cnode"]}')
+            f'{connectURL}?gssession={self.gsdbs.cookiejar.get("session")}.{self.gsdbs.cookiejar.get("signature")}&target={self.target}')
         await self.sio.wait()
